@@ -22,10 +22,10 @@ class FlaxModule(Module):
     init_rngs: tp.Tuple[str, ...]
 
     # dynamic
-    params: tp.Optional[tp.Dict[str, tp.Any]] = types.Parameter.node()
-    batch_stats: tp.Optional[tp.Dict[str, tp.Any]] = types.BatchStat.node()
-    cache: tp.Optional[tp.Dict[str, tp.Any]] = types.Cache.node()
-    variables: tp.Union[tp.Dict[str, tp.Dict[str, tp.Any]], None] = types.Log.node()
+    params_: tp.Optional[tp.Dict[str, tp.Any]] = types.Parameter.node()
+    batch_stats_: tp.Optional[tp.Dict[str, tp.Any]] = types.BatchStat.node()
+    cache_: tp.Optional[tp.Dict[str, tp.Any]] = types.Cache.node()
+    variables_: tp.Union[tp.Dict[str, tp.Dict[str, tp.Any]], None] = types.Log.node()
     next_key: KeySeq
 
     def __init__(
@@ -35,6 +35,7 @@ class FlaxModule(Module):
         rngs: tp.Sequence[str] = ("dropout",),
         init_rngs: tp.Sequence[str] = ("params",),
         variables: tp.Optional[FrozenDict] = None,
+        method: tp.Optional[str] = None,
     ) -> None:
 
         self.module = to.Hashable(module)
@@ -42,44 +43,58 @@ class FlaxModule(Module):
         self.rngs = tuple(rngs)
         self.init_rngs = tuple(init_rngs)
         self.next_key = KeySeq()
-        self.params = None
-        self.batch_stats = None
-        self.cache = None
-        self.variables = None
+        self.params_ = None
+        self.batch_stats_ = None
+        self.cache_ = None
+        self.variables_ = None
+        self.method = method if method is not None else "__call__"
 
         if variables is not None:
             self._update_variables(variables)
 
     def __call__(self, *args, **kwargs):
 
-        if not self.initialized and self.variables is None:
+        method: tp.Callable = getattr(self.module.value, self.method)
+
+        if "training" not in kwargs:
+            arg_names = utils._function_argument_names(method)
+
+            if arg_names is not None and "training" in arg_names:
+                kwargs["training"] = self.training if self.initialized else False
+
+        if self.initializing() and self.variables_ is None:
             rngs = self._get_rngs(self.rngs + self.init_rngs)
-            _variables = self.module.value.init(
+            output, _variables = self.module.value.init_with_output(
                 rngs,
                 *args,
+                method=method,
                 **kwargs,
             )
             self._update_variables(_variables)
+            return output
 
-        assert self.variables is not None
-        variables = self.variables.copy()
+        assert self.variables_ is not None
+        variables = self.variables_.copy()
 
-        if self.params is not None:
-            variables["params"] = self.params
+        if self.params_ is not None:
+            variables["params"] = self.params_
 
-        if self.batch_stats is not None:
-            variables["batch_stats"] = self.batch_stats
+        if self.batch_stats_ is not None:
+            variables["batch_stats"] = self.batch_stats_
 
-        if self.cache is not None:
-            variables["cache"] = self.cache
+        if self.cache_ is not None:
+            variables["cache"] = self.cache_
 
         rngs = self._get_rngs(self.rngs)
 
         output, updates = self.module.value.apply(
             variables,
             *args,
-            mutable=self.mutable if self.initialized else [],
+            mutable=self.mutable
+            if self.initialized and self.training and not self.frozen
+            else [],
             rngs=rngs,
+            method=method,
             **kwargs,
         )
         variables.update(updates.unfreeze())
@@ -115,12 +130,12 @@ class FlaxModule(Module):
         variables = tp.cast(tp.Dict[str, tp.Dict[str, tp.Any]], variables)
 
         if "params" in variables:
-            self.params = variables.pop("params")
+            self.params_ = variables.pop("params")
 
         if "batch_stats" in variables:
-            self.batch_stats = variables.pop("batch_stats")
+            self.batch_stats_ = variables.pop("batch_stats")
 
         if "cache" in variables:
-            self.cache = variables.pop("cache")
+            self.cache_ = variables.pop("cache")
 
-        self.variables = variables
+        self.variables_ = variables
